@@ -11,6 +11,7 @@
  *    Save image
  *
  * Copyright 2000, Helixcode Inc.
+ * Copyright 2000, Eazel, Inc.
  */
 #include <config.h>
 #include <stdio.h>
@@ -20,7 +21,12 @@
 #include <errno.h>
 
 #include <gnome.h>
+#if USING_OAF
+#include <liboaf/liboaf.h>
+#else
 #include <libgnorba/gnorba.h>
+#endif
+
 #include <bonobo.h>
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
@@ -31,6 +37,8 @@
 #include <libart_lgpl/art_rgb_pixbuf_affine.h>
 #include <libart_lgpl/art_alphagamma.h>
 
+#include "io-png.h"
+
 #undef EOG_DEBUG
 
 /*
@@ -130,18 +138,27 @@ render_pixbuf (GdkPixbuf *buf, GtkWidget *dest_widget,
 {
 	g_return_if_fail (buf != NULL);
 
-	/* No drawing area yet ! */
-	if (!dest_widget || !dest_widget->window)
+	if (!GTK_IS_DRAWING_AREA (dest_widget)) {
+		g_warning ("Non drawing area widget");
 		return;
+	}
+
+	/* No drawing area yet ! */
+	if (!dest_widget || !dest_widget->window) {
+/*		g_warning ("No window drawable");*/
+		return;
+	}
 
 	/*
 	 * Do not draw outside the region that we know how to display
 	 */
-	if (rect->x > gdk_pixbuf_get_width (buf))
-		return;
-
-	if (rect->y > gdk_pixbuf_get_height (buf))
+	if (rect->x > gdk_pixbuf_get_width (buf) ||
+	    rect->y > gdk_pixbuf_get_height (buf)) {
+/*		g_warning ("Render outside range %d %d %d %d (%d, %d)", rect->x, rect->y,
+			   gdk_pixbuf_get_width (buf), gdk_pixbuf_get_height (buf),
+			   rect->width, rect->height);*/
 		return;
+	}
 
 	/*
 	 * Clip the draw region
@@ -152,9 +169,7 @@ render_pixbuf (GdkPixbuf *buf, GtkWidget *dest_widget,
 	if (rect->y + rect->height > gdk_pixbuf_get_height (buf))
 		rect->height = gdk_pixbuf_get_height (buf) - rect->y;
 
-	/*
-	 * Draw the exposed region.
-	 */
+	/* Draw into the exposed region. */
 	if (gdk_pixbuf_get_has_alpha (buf))
 		gdk_draw_rgb_32_image (dest_widget->window,
 				       dest_widget->style->white_gc,
@@ -189,9 +204,10 @@ redraw_view (view_data_t *view_data, GdkRectangle *rect)
 	 * so we don't screw up the size allocation process by drawing
 	 * an unscaled image too early.
 	 */
-
 	if (view_data->size_allocated)
 	        render_pixbuf (buf, view_data->drawing_area, rect);
+	else
+		g_warning ("No render, size not allocated");
 }
 
 static void
@@ -203,10 +219,10 @@ configure_size (view_data_t *view_data, GdkRectangle *rect)
 	g_return_if_fail (view_data != NULL);
 
 	/*
-	 * Don't configure the size if it hsan't gotten allocated, to
+	 * Don't configure the size if it hasn't gotten allocated, to
 	 * avoid messing with size_allocate process.
 	 */
-	if (view_data->size_allocated) {
+	if (!view_data->size_allocated) {
 		gtk_widget_set_usize (view_data->drawing_area,
 				      gdk_pixbuf_get_width (buf),
 				      gdk_pixbuf_get_height (buf));
@@ -215,22 +231,30 @@ configure_size (view_data_t *view_data, GdkRectangle *rect)
 		rect->y = 0;
 		rect->width  = gdk_pixbuf_get_width (buf);
 		rect->height = gdk_pixbuf_get_height (buf);
+
+		view_data->size_allocated = TRUE;
+		g_warning ("Configure size; allocate %d %d",
+			   gdk_pixbuf_get_width (buf),
+			   gdk_pixbuf_get_height (buf));
+	} else {
+		GtkAllocation *a = &view_data->drawing_area->allocation;
+		rect->x = a->x;
+		rect->y = a->y;
+		rect->width  = a->width;
+		rect->height = a->height;
 	}
 }
 
 static void
-redraw_all_cb (BonoboView *view, void *data)
+resize_all_cb (BonoboView *view, void *data)
 {
-	GdkRectangle rect;
-	view_data_t *view_data;
+	GtkWidget *widget;
 
 	g_return_if_fail (view != NULL);
 
-	view_data = gtk_object_get_data (GTK_OBJECT (view),
-					 "view_data");
-	configure_size (view_data, &rect);
-	
-	redraw_view (view_data, &rect);
+	widget = bonobo_control_get_widget (BONOBO_CONTROL (view));
+
+	gtk_widget_queue_resize (widget);
 }
 
 static void
@@ -246,55 +270,75 @@ view_update (view_data_t *view_data)
 }
 
 /*
- * Loads an Image from a Bonobo_Stream
+ * Loads a png to a Bonobo_Stream
  */
-static int
-load_image_from_stream (BonoboPersistStream *ps, Bonobo_Stream stream, void *data)
+static void
+save_image_to_stream (BonoboPersistStream *ps, Bonobo_Stream stream,
+		      Bonobo_Persist_ContentType type, void *data,
+		      CORBA_Environment *ev)
 {
-	int                   retval = 0;
 	bonobo_object_data_t *bod = data;
-	GdkPixbufLoader      *loader = gdk_pixbuf_loader_new ();
-	Bonobo_Stream_iobuf   *buffer;
-	CORBA_Environment     ev;
-
-	CORBA_exception_init (&ev);
-
-	do {
-		buffer = Bonobo_Stream_iobuf__alloc ();
-		Bonobo_Stream_read (stream, 4096, &buffer, &ev);
-		if (buffer->_buffer &&
-		    (ev._major != CORBA_NO_EXCEPTION ||
-		     !gdk_pixbuf_loader_write (loader,
-					       buffer->_buffer,
-					       buffer->_length))) {
-			if (ev._major != CORBA_NO_EXCEPTION)
-				g_warning ("Fatal error loading from stream");
-			else
-				g_warning ("Fatal image format error");
-
-			gdk_pixbuf_loader_close (loader);
-			gtk_object_destroy (GTK_OBJECT (loader));
-			CORBA_free (buffer);
-			CORBA_exception_free (&ev);
-			return -1;
-		}
-		CORBA_free (buffer);
-	} while (buffer->_length > 0);
-
-	bod->pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
-	gdk_pixbuf_ref (bod->pixbuf);
-	gdk_pixbuf_loader_close (loader);
-	gtk_object_destroy (GTK_OBJECT (loader));
 
 	if (!bod->pixbuf)
-		retval = -1;
-	else
-		bonobo_embeddable_foreach_view (bod->bonobo_object,
-						redraw_all_cb, bod);
-	
-	CORBA_exception_free (&ev);
+		return;
 
-	return retval;
+	image_save (stream, bod->pixbuf, ev);
+}
+
+/*
+ * Loads an Image from a Bonobo_Stream
+ */
+static void
+load_image_from_stream (BonoboPersistStream *ps, Bonobo_Stream stream,
+			Bonobo_Persist_ContentType type, void *data,
+			CORBA_Environment *ev)
+{
+	bonobo_object_data_t *bod = data;
+	GdkPixbufLoader      *loader = gdk_pixbuf_loader_new ();
+	Bonobo_Stream_iobuf  *buffer;
+	CORBA_long            len;
+
+	do {
+		Bonobo_Stream_read (stream, 4096, &buffer, ev);
+		if (ev->_major != CORBA_NO_EXCEPTION)
+			goto exit_clean;
+
+		if (buffer->_buffer &&
+		     !gdk_pixbuf_loader_write (loader,
+					       buffer->_buffer,
+					       buffer->_length)) {
+			CORBA_free (buffer);
+			if (ev->_major == CORBA_NO_EXCEPTION)
+				goto exit_clean;
+			else
+				goto exit_wrong_type;
+		}
+		len = buffer->_length;
+
+		CORBA_free (buffer);
+	} while (len > 0);
+
+	bod->pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
+
+	if (!bod->pixbuf)
+		goto exit_wrong_type;
+
+	else {
+		gdk_pixbuf_ref (bod->pixbuf);
+		bonobo_embeddable_foreach_view (bod->bonobo_object,
+						resize_all_cb, bod);
+	}
+
+	goto exit_clean;
+
+ exit_wrong_type:
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
+			     ex_Bonobo_Persist_WrongDataType, NULL);
+
+ exit_clean:
+	gdk_pixbuf_loader_close (loader);
+	gtk_object_unref (GTK_OBJECT (loader));
+	return;
 }
 
 static void
@@ -306,14 +350,23 @@ destroy_view (BonoboView *view, view_data_t *view_data)
 		gdk_pixbuf_unref (view_data->scaled);
 	view_data->scaled = NULL;
 
+	gtk_widget_destroy (view_data->drawing_area);
+	view_data->drawing_area = NULL;
+
+	if (view_data->scrolled_window)
+		gtk_widget_destroy (view_data->scrolled_window);
+	view_data->scrolled_window = NULL;
+
 	g_free (view_data);
 }
 
 static int
 drawing_area_exposed (GtkWidget *widget, GdkEventExpose *event, view_data_t *view_data)
 {
-	if (!view_data->bod->pixbuf)
+	if (!view_data->bod->pixbuf) {
+/*		g_warning ("Exposed with no pixbuf");*/
 		return TRUE;
+	}
 	
 	redraw_view (view_data, &event->area);
 
@@ -329,6 +382,7 @@ view_size_allocate_cb (GtkWidget *drawing_area, GtkAllocation *allocation,
 {
 	const GdkPixbuf *buf;
 	GdkPixbuf       *view_buf;
+	GdkInterpType    type;
 
 	g_return_if_fail (view_data != NULL);
 	g_return_if_fail (allocation != NULL);
@@ -370,12 +424,20 @@ view_size_allocate_cb (GtkWidget *drawing_area, GtkAllocation *allocation,
 	}
 
 #ifdef EOG_DEBUG
-	g_warning ("Re-scale");
+	g_warning ("Re-scale to %d, %d", allocation->width, allocation->height);
 #endif
-	/* FIXME: should we be FILTER_TILES / FILTER_NEAREST ? */
+	/* Too slow below this */
+	if (allocation->width < gdk_pixbuf_get_width (buf) / 4 ||
+	    allocation->width < gdk_pixbuf_get_width (buf) / 4)
+		type = ART_FILTER_NEAREST;
+	else
+		type = ART_FILTER_TILES;
+
 	view_data->scaled = gdk_pixbuf_scale_simple (buf, allocation->width,
-						     allocation->height,
-						     ART_FILTER_TILES);
+						     allocation->height, type);
+#ifdef EOG_DEBUG
+	g_warning ("Scaling done");
+#endif
 	view_update (view_data);
 }
 
@@ -435,7 +497,7 @@ view_factory_common (BonoboEmbeddable *bonobo_object,
 	view_data->bod = bod;
 	view_data->scaled = NULL;
 	view_data->drawing_area = gtk_drawing_area_new ();
-	view_data->size_allocated = TRUE;
+	view_data->size_allocated = FALSE;
 	view_data->scrolled_window = scrolled_window;
 
 	gtk_signal_connect (
@@ -459,6 +521,8 @@ view_factory_common (BonoboEmbeddable *bonobo_object,
 	gtk_signal_connect (GTK_OBJECT (view), "destroy",
 			    GTK_SIGNAL_FUNC (destroy_view), view_data);
 
+	running_objects++;
+
         return view;
 }
 
@@ -523,10 +587,19 @@ bonobo_object_factory (BonoboGenericFactory *this, const char *goad_id, void *da
 	 * Creates the BonoboObject server
 	 */
 
+#if USING_OAF
+	if (!strcmp (goad_id, "OAFIID:eog_image-generic:0d77ee99-ce0d-4463-94ec-99969f567f33"))
+#else
 	if (!strcmp (goad_id, "embeddable:image-generic"))
+#endif
 		bonobo_object = bonobo_embeddable_new (scaled_view_factory, bod);
 
+
+#if USING_OAF
+	else if (!strcmp (goad_id, "OAFIID:eog-image-viewer:f67c01c8-a44d-4c50-8ce1-dc893c961876"))
+#else
 	else if (!strcmp (goad_id, "eog-image-viewer"))
+#endif
 		bonobo_object = bonobo_embeddable_new (scrollable_view_factory, bod);
 
 	else {
@@ -544,8 +617,9 @@ bonobo_object_factory (BonoboGenericFactory *this, const char *goad_id, void *da
 	/*
 	 * Interface Bonobo::PersistStream 
 	 */
-	stream = bonobo_persist_stream_new (load_image_from_stream,
-					    NULL, bod);
+	stream = bonobo_persist_stream_new (load_image_from_stream, 
+					    save_image_to_stream, 
+					    NULL, NULL, bod);
 	if (stream == NULL) {
 		gtk_object_unref (GTK_OBJECT (bonobo_object));
 		g_free (bod);
@@ -575,9 +649,15 @@ bonobo_object_factory (BonoboGenericFactory *this, const char *goad_id, void *da
 static void
 init_bonobo_image_generic_factory (void)
 {
+#if USING_OAF
 	image_factory = bonobo_generic_factory_new_multi 
-		("embeddable-factory:image-generic",
+		("OAFIID:eog_viewer_factory:777e0cdf-2b79-4e36-93d8-e9d490c9c4b8",
 		 bonobo_object_factory, NULL);
+#else
+        image_factory = bonobo_generic_factory_new_multi 
+	        ("embeddable-factory:image-generic",
+		 bonobo_object_factory, NULL);
+#endif
 }
 
 static void
@@ -586,9 +666,16 @@ init_server_factory (int argc, char **argv)
 	CORBA_Environment ev;
 	CORBA_exception_init (&ev);
 
+#ifdef USING_OAF
+        gnome_init_with_popt_table("bonobo-image-generic", VERSION,
+				   argc, argv,
+				   oaf_popt_options, 0, NULL); 
+	oaf_init (argc, argv);
+#else
 	gnome_CORBA_init_with_popt_table (
 		"bonobo-image-generic", "1.0",
 		&argc, argv, NULL, 0, NULL, GNORBA_INIT_SERVER_FUNC, &ev);
+#endif
 
 	if (bonobo_init (CORBA_OBJECT_NIL, CORBA_OBJECT_NIL, CORBA_OBJECT_NIL) == FALSE)
 		g_error (_("I could not initialize Bonobo"));
@@ -604,7 +691,7 @@ typedef struct {
 	GdkPixbufAnimation *animation;
 	GdkPixbuf          *frame;
 	GList              *cur_frame;
-	guint               timeout;
+	gint                timeout;
 	
 	gboolean            animate;
 	char               *fname;
@@ -621,7 +708,23 @@ animation_state_clean (AnimationState *as)
 		gdk_pixbuf_unref (as->frame);
 	as->frame = NULL;
 
-	as->cur_frame    = NULL;
+	if (as->timeout >= 0)
+		gtk_timeout_remove (as->timeout);
+	as->timeout = -1;
+
+	as->cur_frame = NULL;
+}
+
+static GdkPixbufFrame *
+animation_cur_frame (AnimationState *as)
+{
+	if (!as->cur_frame)
+		as->cur_frame = gdk_pixbuf_animation_get_frames (as->animation);
+
+	if (!as->cur_frame)
+		return NULL;
+
+	return as->cur_frame->data;
 }
 
 static gint
@@ -636,12 +739,12 @@ skip_frame (AnimationState *as)
 	if (!as->animate)
 		return FALSE;
 
-	if (!as->cur_frame) {
+	frame = animation_cur_frame (as);
+	if (!frame) {
 		g_warning ("No frame to render");
 		return FALSE;
 	}
 
-	frame = as->cur_frame->data;
 	g_return_val_if_fail (frame != NULL, FALSE);
 	g_return_val_if_fail (gdk_pixbuf_frame_get_pixbuf (frame) != NULL, FALSE);
 
@@ -664,6 +767,11 @@ skip_frame (AnimationState *as)
 
 	layer = gdk_pixbuf_frame_get_pixbuf (frame);
 
+/*
+ * NB. None of this is used, the expose event just renders the raw
+ * frame pixbuf; since this code looks nice but is broken.
+ */
+
 	if (!as->cur_frame->prev ||
 	    gdk_pixbuf_frame_get_action (frame) == GDK_PIXBUF_FRAME_REVERT) {
 		gdk_pixbuf_copy_area (layer, 0, 0, w, h, as->frame,
@@ -685,10 +793,9 @@ skip_frame (AnimationState *as)
 				    w, h);
 	as->timeout = gtk_timeout_add (gdk_pixbuf_frame_get_delay_time (frame) * 10,
 				       (GtkFunction) skip_frame, as);
+/*	g_warning ("Delay %d\n", gdk_pixbuf_frame_get_delay_time (frame) * 10);*/
 
 	as->cur_frame = as->cur_frame->next;
-	if (!as->cur_frame)
-		as->cur_frame = gdk_pixbuf_animation_get_frames (as->animation);
 
 	return FALSE;
 }
@@ -702,7 +809,10 @@ animation_init (AnimationState *as, char *fname)
 	if (as->animation) {
 		GdkPixbufFrame *frame;
 
-		as->cur_frame = gdk_pixbuf_animation_get_frames (as->animation);
+		/*
+		 * So, as we skip onto the next frame we start at the beggining.
+		 */
+		as->cur_frame = g_list_last (gdk_pixbuf_animation_get_frames (as->animation));
 		g_return_if_fail (as->cur_frame != NULL);
 
 		frame = as->cur_frame->data;
@@ -719,8 +829,12 @@ animation_init (AnimationState *as, char *fname)
 		gtk_widget_set_usize (as->drawing_area,
 				      gdk_pixbuf_get_width  (as->frame),
 				      gdk_pixbuf_get_height (as->frame));
-	} else
+	}
+#ifdef EOG_DEBUG
+	/* As you type into glade you get this a lot */
+	else
 		g_warning ("Error loading animation '%s'", fname);
+#endif
 }
 
 static void
@@ -731,8 +845,6 @@ animation_destroy (BonoboView *view, AnimationState *as)
 	animation_state_clean (as);
 	as->drawing_area = NULL;
 
-	gtk_timeout_remove (as->timeout);
-
 	if (as->fname)
 		g_free (as->fname);
 	as->fname = NULL;
@@ -751,12 +863,15 @@ animation_area_exposed (GtkWidget *widget, GdkEventExpose *event,
 	if (!as->frame)
 		return TRUE;
 
-	if (!as->cur_frame && !as->cur_frame->data)
+/*
+ * FIXME: herin lies the problem ... this needs to be fixed
+ * should use a nicely composited as->frame.
+ */
+	frame = animation_cur_frame (as);
+	if (!frame) {
+		g_warning ("No frame to render");
 		return TRUE;
-
-/*	render_pixbuf (as->frame, as->drawing_area, &event->area);*/
-/* FIXME: herin lies the problem ... this needs to be fixed */
-	frame = as->cur_frame->data;
+	}
 
 	render_pixbuf (gdk_pixbuf_frame_get_pixbuf (frame), as->drawing_area, &event->area);
 
@@ -828,6 +943,8 @@ bonobo_animator_factory (BonoboGenericFactory *Factory, void *closure)
 
 	as = g_new0 (AnimationState, 1);
 	as->fname = NULL;
+	as->timeout = -1;
+	as->animate = TRUE;
 
 	/* Create the control. */
 	as->drawing_area = gtk_drawing_area_new ();
@@ -865,8 +982,13 @@ init_bonobo_animator_control_factory (void)
 	if (animator_factory != NULL)
 		return;
 
+#if USING_OAF
+	animator_factory = bonobo_generic_factory_new ("OAFIID:eog_animator_factory:8d2380ad-6e60-4986-ae27-152501839f57",
+						       bonobo_animator_factory, NULL);
+#else
 	animator_factory = bonobo_generic_factory_new ("control-factory:animator",
 						       bonobo_animator_factory, NULL);
+#endif
 
 	if (animator_factory == NULL)
 		g_error ("I could not register an animator factory.");
