@@ -1,8 +1,8 @@
 /* Eye of Gnome image viewer - user interface for image views
  *
- * Copyright (C) 1999 The Free Software Foundation
+ * Copyright (C) 2000 The Free Software Foundation
  *
- * Author: Federico Mena-Quintero <federico@gimp.org>
+ * Author: Federico Mena-Quintero <federico@gnu.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,15 +52,14 @@ struct _UIImageClass {
 };
 
 
+void ui_image_set_scroll_policy (UIImage *ui, GtkPolicyType policy);
 GtkType ui_image_get_type (void);
 
 GtkWidget *ui_image_new (void);
 GtkWidget *ui_image_construct (UIImage *ui);
 
-void ui_image_set_image (UIImage *ui, Image *image);
+GtkWidget *ui_image_get_image_view (UIImage *ui);
 
-void ui_image_set_zoom (UIImage *ui, double zoom);
-double ui_image_get_zoom (UIImage *ui);
 void ui_image_zoom_fit (UIImage *ui);
 
 
