@@ -1,8 +1,8 @@
 /* Eye of Gnome image viewer - main window widget
  *
- * Copyright (C) 1999 The Free Software Foundation
+ * Copyright (C) 2000 The Free Software Foundation
  *
- * Author: Federico Mena-Quintero <federico@gimp.org>
+ * Author: Federico Mena-Quintero <federico@gnu.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,86 +20,61 @@
  */
 
 #include <config.h>
+#include <math.h>
 #include <gnome.h>
 #include "commands.h"
+#include "full-screen.h"
+#include "image-view.h"
+#include "preferences.h"
 #include "tb-image.h"
 #include "ui-image.h"
 #include "util.h"
 #include "window.h"
-
-
+#include "zoom.h"
 
 /* Default size for windows */
 
 #define DEFAULT_WINDOW_WIDTH 400
 #define DEFAULT_WINDOW_HEIGHT 300
 
-
-
-/* What a window is displaying */
-typedef enum {
-	WINDOW_MODE_NONE,
-	WINDOW_MODE_IMAGE,
-	WINDOW_MODE_COLLECTION
-} WindowMode;
-
 /* Private part of the Window structure */
 typedef struct {
-	/* What we are displaying */
-	WindowMode mode;
-
-	/* Our contents, which may change depending on the mode */
-	GtkWidget *content;
+	/* UIImage widget for our contents */
+	GtkWidget *ui;
 
 	/* The file selection widget used by this window.  We keep it around so
 	 * that it will preserve its cwd.
 	 */
 	GtkWidget *file_sel;
 
+	/* Our GConf client */
+	GConfClient *client;
+
 	/* View menu */
 	GtkWidget *view_menu;
 
 	/* Zoom toolbar items */
 	GtkWidget **zoom_tb_items;
-} WindowPrivate;
 
-
+	/* Window scrolling policy type */
+	GtkPolicyType sb_policy;
+
+	/* GConf client notify id's */
+	guint sb_policy_notify_id;
+} WindowPrivate;
 
 static void window_class_init (WindowClass *class);
 static void window_init (Window *window);
 static void window_destroy (GtkObject *object);
 
 static gint window_delete (GtkWidget *widget, GdkEventAny *event);
-
+static gint window_key_press (GtkWidget *widget, GdkEventKey *event);
 
 static GnomeAppClass *parent_class;
 
-
-
-/* What the user may respond to a close window confirmation */
-typedef enum {
-	CLOSE_SAVE,
-	CLOSE_DISCARD,
-	CLOSE_DONT_EXIT
-} CloseStatus;
-
 /* The list of all open windows */
 static GList *window_list;
 
-/* Returns whether a window contains unsaved data */
-static gboolean
-is_unsaved (Window *window)
-{
-	WindowPrivate *priv;
-
-	priv = window->priv;
-
-	if (priv->mode == WINDOW_MODE_COLLECTION)
-		return TRUE; /* FIXME: is it dirty? */
-	else
-		return FALSE;
-}
-
 /* Brings attention to a window by raising it and giving it focus */
 static void
 raise_and_focus (GtkWidget *widget)
@@ -109,142 +84,6 @@ raise_and_focus (GtkWidget *widget)
 	gtk_widget_grab_focus (widget);
 }
 
-/* Asks for confirmation for closing an unsaved window */
-static CloseStatus
-confirm_save (Window *window, gboolean ask_exit)
-{
-	WindowPrivate *priv;
-	GtkWidget *msg;
-	char *buf;
-	int result;
-
-	priv = window->priv;
-
-	if (!is_unsaved (window))
-		return CLOSE_DISCARD;
-
-	g_assert (priv->mode == WINDOW_MODE_COLLECTION);
-
-	raise_and_focus (GTK_WIDGET (window));
-
-	buf = g_strdup_printf (_("Save image collection `%s'?"),
-			       "fubari"); /* FIXME: put in title */
-
-	msg = gnome_message_box_new (buf, GNOME_MESSAGE_BOX_QUESTION,
-				     _("Save"),
-				     _("Don't save"),
-				     ask_exit ? _("Don't exit") : NULL,
-				     NULL);
-	g_free (buf);
-
-	gnome_dialog_set_parent (GNOME_DIALOG (msg), GTK_WINDOW (window));
-	gnome_dialog_set_default (GNOME_DIALOG (msg), 0);
-
-	result = gnome_dialog_run (GNOME_DIALOG (msg));
-
-	switch (result) {
-	case 0:
-		return CLOSE_SAVE;
-
-	case 1:
-		return CLOSE_DISCARD;
-
-	case 2:
-		return CLOSE_DONT_EXIT;
-
-	default:
-		return CLOSE_DONT_EXIT;
-	}
-}
-
-
-
-/* Setting the mode of a window */
-
-static void
-sensitize_zoom_items (GtkWidget **widgets, gboolean sensitive)
-{
-	g_assert (widgets != NULL);
-
-	for (; *widgets != NULL; widgets++)
-		gtk_widget_set_sensitive (*widgets, sensitive);
-}
-
-/* Sets the sensitivity of menu items according to the mode */
-static void
-set_menu_tb_sensitivity (Window *window)
-{
-	WindowPrivate *priv;
-
-	priv = window->priv;
-
-	switch (priv->mode) {
-	case WINDOW_MODE_NONE:
-		gtk_widget_set_sensitive (priv->view_menu, FALSE);
-		sensitize_zoom_items (priv->zoom_tb_items, FALSE);
-		break;
-
-	case WINDOW_MODE_IMAGE:
-		gtk_widget_set_sensitive (priv->view_menu, TRUE);
-		sensitize_zoom_items (priv->zoom_tb_items, TRUE);
-		break;
-
-	case WINDOW_MODE_COLLECTION:
-		gtk_widget_set_sensitive (priv->view_menu, FALSE);
-		sensitize_zoom_items (priv->zoom_tb_items, FALSE);
-		/* FIXME: finish this */
-		break;
-
-	default:
-		g_assert_not_reached ();
-	}
-}
-
-/* Sets the mode of a window */
-static void
-set_mode (Window *window, WindowMode mode)
-{
-	WindowPrivate *priv;
-
-	priv = window->priv;
-
-	/* FIXME: this test may be better outside this function */
-	switch (confirm_save (window, FALSE)) {
-	case CLOSE_SAVE:
-		; /* FIXME: save it */
-		break;
-
-	default:
-		break;
-	}
-
-	if (priv->content)
-		gtk_widget_destroy (priv->content);
-
-	priv->mode = mode;
-
-	switch (mode) {
-	case WINDOW_MODE_NONE:
-		break;
-
-	case WINDOW_MODE_IMAGE:
-		priv->content = ui_image_new ();
-		gnome_app_set_contents (GNOME_APP (window), priv->content);
-		gtk_widget_show (priv->content);
-		break;
-
-	case WINDOW_MODE_COLLECTION:
-		/* FIXME - create UI, fallthrough */
-
-	default:
-		g_assert_not_reached ();
-	}
-
-	set_menu_tb_sensitivity (window);
-}
-
-
-
 /* Creating a new window */
 
 /* Creates a new window */
@@ -257,8 +96,6 @@ create_window (void)
 	gtk_widget_show (window);
 }
 
-
-
 /* File/New Window */
 
 /* File/New Window callback */
@@ -268,46 +105,20 @@ new_window_cmd (GtkWidget *widget, gpointer data)
 	create_window ();
 }
 
-
-
 /* File/Close and Exit */
 
 /* File/Exit callback */
 static void
 exit_cmd (GtkWidget *widget, gpointer data)
 {
-	gboolean do_exit;
 	GList *l;
-	Window *w;
-
-	/* Ask for confirmation on unsaved windows */
-
-	do_exit = TRUE;
-
-	for (l = window_list; l && do_exit; l = l->next) {
-		w = l->data;
-
-		switch (confirm_save (w, TRUE)) {
-		case CLOSE_SAVE:
-			; /* FIXME: save it */
-			break;
-
-		case CLOSE_DONT_EXIT:
-			do_exit = FALSE;
-			break;
-
-		default:
-			break;
-		}
-	}
-
-	if (!do_exit)
-		return;
 
 	/* Destroy windows and exit */
 
 	l = window_list;
 	while (l) {
+		Window *w;
+
 		w = l->data;
 		l = l->next;
 		gtk_widget_destroy (GTK_WIDGET (w));
@@ -316,7 +127,12 @@ exit_cmd (GtkWidget *widget, gpointer data)
 	gtk_main_quit ();
 }
 
-
+/* Settings/Preferences callback */
+static void
+preferences_cmd (GtkWidget *widget, gpointer data)
+{
+	prefs_dialog ();
+}
 
 /* Help/About */
 
@@ -326,7 +142,8 @@ about_cmd (GtkWidget *widget, gpointer data)
 {
 	static GtkWidget *about;
 	static const char *authors[] = {
-		"Federico Mena-Quintero (federico@gimp.org)",
+		"Federico Mena-Quintero (federico@gnu.org)",
+		"Arik Devens (arik@helixcode.com)",
 		NULL
 	};
 
@@ -334,7 +151,7 @@ about_cmd (GtkWidget *widget, gpointer data)
 		about = gnome_about_new (
 			_("Eye of Gnome"),
 			VERSION,
-			_("Copyright (C) 1999 The Free Software Foundation"),
+			_("Copyright (C) 2000 The Free Software Foundation"),
 			authors,
 			_("The GNOME image viewing and cataloging program"),
 			NULL);
@@ -347,8 +164,6 @@ about_cmd (GtkWidget *widget, gpointer data)
 	raise_and_focus (about);
 }
 
-
-
 /* Main menu */
 
 static GnomeUIInfo file_menu[] = {
@@ -362,6 +177,8 @@ static GnomeUIInfo file_menu[] = {
 	  cmd_cb_window_close, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE,
 	  'w', GDK_CONTROL_MASK, NULL },
+
+#define FILE_EXIT_INDEX 4
 	GNOMEUIINFO_MENU_EXIT_ITEM (exit_cmd, NULL),
 	GNOMEUIINFO_END
 };
@@ -391,11 +208,11 @@ static GnomeUIInfo zoom_menu[] = {
 };
 
 static GnomeUIInfo view_menu[] = {
-	{ GNOME_APP_UI_ITEM, N_("Zoom In"), N_("Increase zoom factor by 5%"),
+	{ GNOME_APP_UI_ITEM, N_("Zoom In"), N_("Increase zoom factor by 5%%"),
 	  cmd_cb_zoom_in, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  '=', 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("Zoom Out"), N_("Decrease zoom factor by 5%"),
+	{ GNOME_APP_UI_ITEM, N_("Zoom Out"), N_("Decrease zoom factor by 5%%"),
 	  cmd_cb_zoom_out, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  '-', 0, NULL },
@@ -409,6 +226,15 @@ static GnomeUIInfo view_menu[] = {
 	  cmd_cb_zoom_fit, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  'f', 0, NULL },
+	{ GNOME_APP_UI_ITEM, N_("Full _Screen"), N_("Use the whole screen for display"),
+	  cmd_cb_full_screen, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  's', 0, NULL },
+	GNOMEUIINFO_END
+};
+
+static GnomeUIInfo settings_menu[] = {
+	GNOMEUIINFO_MENU_PREFERENCES_ITEM (preferences_cmd, NULL),
 	GNOMEUIINFO_END
 };
 
@@ -420,12 +246,11 @@ static GnomeUIInfo help_menu[] = {
 static GnomeUIInfo main_menu[] = {
 	GNOMEUIINFO_MENU_FILE_TREE (file_menu),
 	GNOMEUIINFO_MENU_VIEW_TREE (view_menu),
+	GNOMEUIINFO_MENU_SETTINGS_TREE (settings_menu),
 	GNOMEUIINFO_MENU_HELP_TREE (help_menu),
 	GNOMEUIINFO_END
 };
 
-
-
 /**
  * window_get_type:
  * @void:
@@ -473,6 +298,23 @@ window_class_init (WindowClass *class)
 	object_class->destroy = window_destroy;
 
 	widget_class->delete_event = window_delete;
+	widget_class->key_press_event = window_key_press;
+}
+
+/* Handler for changes on the window sb policy */
+static void
+sb_policy_changed_cb (GConfClient *client, guint notify_id, const gchar *key,
+		      GConfValue *value, gboolean is_default, gpointer data)
+{
+	Window *window;
+	WindowPrivate *priv;
+
+	window = WINDOW (data);
+	priv = window->priv;
+
+	priv->sb_policy = gconf_value_int (value);
+
+	gtk_scroll_frame_set_policy (GTK_SCROLL_FRAME (priv->ui), priv->sb_policy, priv->sb_policy);
 }
 
 /* Object initialization function for windows */
@@ -484,7 +326,23 @@ window_init (Window *window)
 	priv = g_new0 (WindowPrivate, 1);
 	window->priv = priv;
 
+	priv->client = gconf_client_get_default ();
+
+	gconf_client_add_dir (priv->client, "/apps/eog",
+			      GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
+
+	priv->sb_policy_notify_id = gconf_client_notify_add (
+		priv->client, "/apps/eog/window/sb_policy",
+		sb_policy_changed_cb, window,
+		NULL, NULL);
+
+	priv->sb_policy = gconf_client_get_int (
+		priv->client, "/apps/eog/window/sb_policy",
+		NULL);
+
 	window_list = g_list_prepend (window_list, window);
+
+	gtk_window_set_policy (GTK_WINDOW (window), TRUE, TRUE, FALSE);
 }
 
 /* Destroy handler for windows */
@@ -500,15 +358,32 @@ window_destroy (GtkObject *object)
 	window = WINDOW (object);
 	priv = window->priv;
 
-	if (priv->file_sel)
+	if (priv->file_sel) {
 		gtk_widget_destroy (priv->file_sel);
+		priv->file_sel = NULL;
+	}
 
 	g_free (priv->zoom_tb_items);
-
-	g_free (priv);
+	priv->zoom_tb_items = NULL;
 
 	window_list = g_list_remove (window_list, window);
 
+	/* Remove notification handlers */
+
+	gconf_client_notify_remove (priv->client, priv->sb_policy_notify_id);
+
+	priv->sb_policy_notify_id = 0;
+
+	gconf_client_remove_dir (priv->client, "/apps/eog", NULL);
+
+	gtk_object_unref (GTK_OBJECT (priv->client));
+	priv->client = NULL;
+
+	/* Clean up */
+
+	g_free (priv);
+	window->priv = NULL;
+
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
@@ -524,6 +399,33 @@ window_delete (GtkWidget *widget, GdkEventAny *event)
 	return TRUE;
 }
 
+/* Key press handler for windows */
+static gint
+window_key_press (GtkWidget *widget, GdkEventKey *event)
+{
+	gint result;
+
+	result = FALSE;
+
+	if (GTK_WIDGET_CLASS (parent_class)->key_press_event)
+		result = (* GTK_WIDGET_CLASS (parent_class)->key_press_event) (widget, event);
+
+	if (result)
+		return result;
+
+	switch (event->keyval) {
+	case GDK_Q:
+	case GDK_q:
+		exit_cmd (NULL, widget);
+		break;
+
+	default:
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 /**
  * window_new:
  * @void:
@@ -543,6 +445,16 @@ window_new (void)
 	return GTK_WIDGET (window);
 }
 
+/* Sets the sensitivity of all the items */
+static void
+sensitize_zoom_items (GtkWidget **widgets, gboolean sensitive)
+{
+	g_assert (widgets != NULL);
+
+	for (; *widgets != NULL; widgets++)
+		gtk_widget_set_sensitive (*widgets, sensitive);
+}
+
 /**
  * window_construct:
  * @window: A window widget.
@@ -554,6 +466,7 @@ window_construct (Window *window)
 {
 	WindowPrivate *priv;
 	GtkWidget *tb;
+	ImageView *view;
 
 	g_return_if_fail (window != NULL);
 	g_return_if_fail (IS_WINDOW (window));
@@ -568,14 +481,19 @@ window_construct (Window *window)
 	tb = tb_image_new (window, &priv->zoom_tb_items);
 	gnome_app_set_toolbar (GNOME_APP (window), GTK_TOOLBAR (tb));
 
-	gtk_window_set_default_size (GTK_WINDOW (window),
-				     DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT);
+	priv->ui = ui_image_new ();
+	gnome_app_set_contents (GNOME_APP (window), priv->ui);
+	gtk_scroll_frame_set_policy (GTK_SCROLL_FRAME (priv->ui), priv->sb_policy, priv->sb_policy);
+	gtk_widget_show (priv->ui);
 
-	set_mode (window, WINDOW_MODE_NONE);
+	view = IMAGE_VIEW (ui_image_get_image_view (UI_IMAGE (priv->ui)));
+
+	gtk_widget_grab_focus (GTK_WIDGET (view));
+
+	gtk_widget_set_sensitive (priv->view_menu, TRUE);
+	sensitize_zoom_items (priv->zoom_tb_items, TRUE);
 }
 
-
-
 /**
  * window_close:
  * @window: A window.
@@ -589,34 +507,51 @@ window_close (Window *window)
 	g_return_if_fail (window != NULL);
 	g_return_if_fail (IS_WINDOW (window));
 
-	switch (confirm_save (window, FALSE)) {
-	case CLOSE_SAVE:
-		; /* FIXME: save it */
-		break;
-
-	case CLOSE_DISCARD:
-		break;
-
-	default:
-		return;
-	}
-
 	gtk_widget_destroy (GTK_WIDGET (window));
 
 	if (!window_list)
 		gtk_main_quit ();
 }
 
-
-
 /* Open image dialog */
 
+/* Opens an image in a new window */
+static void
+open_new_window (Window *window, const char *filename)
+{
+	WindowPrivate *priv;
+	ImageView *view;
+	Image *image;
+	GtkWidget *new_window;
+
+	priv = window->priv;
+
+	view = IMAGE_VIEW (ui_image_get_image_view (UI_IMAGE (priv->ui)));
+	image = image_view_get_image (view);
+
+	if (!(image && image->pixbuf))
+		new_window = GTK_WIDGET (window);
+	else
+		new_window = window_new ();
+
+	if (window_open_image (WINDOW (new_window), filename)) {
+		gtk_widget_show_now (new_window);
+		raise_and_focus (new_window);
+	} else {
+		open_failure_dialog (GTK_WINDOW (new_window), filename);
+
+		if (new_window != GTK_WIDGET (window))
+			gtk_widget_destroy (new_window);
+	}
+}
+
 /* OK button callback for the open file selection dialog */
 static void
 open_ok_clicked (GtkWidget *widget, gpointer data)
 {
 	GtkWidget *fs;
 	Window *window;
+	WindowPrivate *priv;
 	char *filename;
 
 	fs = GTK_WIDGET (data);
@@ -624,10 +559,14 @@ open_ok_clicked (GtkWidget *widget, gpointer data)
 	window = WINDOW (gtk_object_get_data (GTK_OBJECT (fs), "window"));
 	g_assert (window != NULL);
 
+	priv = window->priv;
+
 	filename = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
 	gtk_widget_hide (fs);
 
-	if (!window_open_image (window, filename))
+	if (gconf_client_get_bool (priv->client, "/apps/eog/window/open_new_window", NULL))
+		open_new_window (window, filename);
+	else if (!window_open_image (window, filename))
 		open_failure_dialog (GTK_WINDOW (window), filename);
 
 	g_free (filename);
@@ -665,8 +604,11 @@ window_open_image_dialog (Window *window)
 	priv = window->priv;
 
 	if (!priv->file_sel) {
+		GtkAccelGroup *accel_group;
+
 		priv->file_sel = gtk_file_selection_new (_("Open Image"));
 		gtk_window_set_transient_for (GTK_WINDOW (priv->file_sel), GTK_WINDOW (window));
+		gtk_window_set_modal (GTK_WINDOW (priv->file_sel), TRUE);
 		gtk_object_set_data (GTK_OBJECT (priv->file_sel), "window", window);
 
 		gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (priv->file_sel)->ok_button),
@@ -680,13 +622,54 @@ window_open_image_dialog (Window *window)
 		gtk_signal_connect (GTK_OBJECT (priv->file_sel), "delete_event",
 				    GTK_SIGNAL_FUNC (open_delete_event),
 				    window);
+
+		accel_group = gtk_accel_group_new ();
+		gtk_window_add_accel_group (GTK_WINDOW (priv->file_sel), accel_group);
+		gtk_widget_add_accelerator (GTK_FILE_SELECTION (priv->file_sel)->cancel_button,
+					    "clicked",
+					    accel_group,
+					    GDK_Escape,
+					    0, 0);
 	}
 
 	gtk_widget_show_now (priv->file_sel);
 	raise_and_focus (priv->file_sel);
 }
 
-
+/* Picks a reasonable size for the window and zoom factor based on the image size */
+static void
+auto_size (Window *window)
+{
+	WindowPrivate *priv;
+	GtkWidget *view;
+	Image *image;
+	int iwidth, iheight;
+	int swidth, sheight;
+	int zwidth, zheight;
+	double zoom;
+
+	priv = window->priv;
+
+	view = ui_image_get_image_view (UI_IMAGE (priv->ui));
+	image = image_view_get_image (IMAGE_VIEW (view));
+
+	if (!(image && image->pixbuf))
+		return;
+
+	iwidth = gdk_pixbuf_get_width (image->pixbuf);
+	iheight = gdk_pixbuf_get_height (image->pixbuf);
+
+	swidth = gdk_screen_width () * 0.75;
+	sheight = gdk_screen_height () * 0.75;
+
+	zoom = zoom_fit_scale (swidth, sheight, iwidth, iheight, FALSE);
+	image_view_set_zoom (IMAGE_VIEW (view), zoom);
+
+	zwidth = floor (iwidth * zoom + 0.5);
+	zheight = floor (iheight * zoom + 0.5);
+
+	gtk_widget_set_usize (view, zwidth, zheight);
+}
 
 /**
  * window_open_image:
@@ -706,6 +689,7 @@ window_open_image (Window *window, const char *filename)
 	gboolean retval;
 	char *fname;
 	gboolean free_fname;
+	GtkWidget *view;
 
 	g_return_val_if_fail (window != NULL, FALSE);
 	g_return_val_if_fail (IS_WINDOW (window), FALSE);
@@ -713,12 +697,10 @@ window_open_image (Window *window, const char *filename)
 
 	priv = window->priv;
 
-	set_mode (window, WINDOW_MODE_IMAGE);
-	g_assert (priv->content != NULL && IS_UI_IMAGE (priv->content));
-
 	image = image_new ();
 	retval = image_load (image, filename);
-	ui_image_set_image (UI_IMAGE (priv->content), image);
+	view = ui_image_get_image_view (UI_IMAGE (priv->ui));
+	image_view_set_image (IMAGE_VIEW (view), image);
 
 	free_fname = FALSE;
 
@@ -739,129 +721,29 @@ window_open_image (Window *window, const char *filename)
 	if (free_fname)
 		g_free (fname);
 
+	if (gconf_client_get_bool (priv->client, "/apps/eog/window/auto_size", NULL))
+		auto_size (window);
+
 	image_unref (image);
 	return retval;
 }
 
-
-
 /**
- * window_set_zoom:
- * @window: An image window.
- * @zoom: Desired zoom factor.
- * 
- * Sets the zoom factor for an image window.
- **/
-void
-window_set_zoom (Window *window, double zoom)
-{
-	WindowPrivate *priv;
-	UIImage *ui;
-
-	g_return_if_fail (window != NULL);
-	g_return_if_fail (IS_WINDOW (window));
-
-	priv = window->priv;
-	g_return_if_fail (priv->mode == WINDOW_MODE_IMAGE);
-
-	g_assert (priv->content != NULL && IS_UI_IMAGE (priv->content));
-	ui = UI_IMAGE (priv->content);
-
-	ui_image_set_zoom (ui, zoom);
-}
-
-/**
- * window_zoom_in:
- * @window: An image window.
+ * window_get_ui_image:
+ * @window: A window.
  *
- * Zooms in an image window.
- **/
-void
-window_zoom_in (Window *window)
-{
-	WindowPrivate *priv;
-	UIImage *ui;
-
-	g_return_if_fail (window != NULL);
-	g_return_if_fail (IS_WINDOW (window));
-
-	priv = window->priv;
-	g_return_if_fail (priv->mode == WINDOW_MODE_IMAGE);
-
-	g_assert (priv->content != NULL && IS_UI_IMAGE (priv->content));
-	ui = UI_IMAGE (priv->content);
-
-	ui_image_set_zoom (ui, ui_image_get_zoom (ui) * 1.05);
-}
-
-/**
- * window_zoom_out:
- * @window: An image window.
+ * Queries the image view scroller inside a window.
  *
- * Zooms out an image window.
+ * Return value: An image view scroller.
  **/
-void
-window_zoom_out (Window *window)
+GtkWidget *
+window_get_ui_image (Window *window)
 {
 	WindowPrivate *priv;
-	UIImage *ui;
 
-	g_return_if_fail (window != NULL);
-	g_return_if_fail (IS_WINDOW (window));
+	g_return_val_if_fail (window != NULL, NULL);
+	g_return_val_if_fail (IS_WINDOW (window), NULL);
 
 	priv = window->priv;
-	g_return_if_fail (priv->mode == WINDOW_MODE_IMAGE);
-
-	g_assert (priv->content != NULL && IS_UI_IMAGE (priv->content));
-	ui = UI_IMAGE (priv->content);
-
-	ui_image_set_zoom (ui, ui_image_get_zoom (ui) / 1.05);
-}
-
-/**
- * window_zoom_1:
- * @window: An image window.
- * 
- * Zooms the image to 1:1 scale.
- **/
-void
-window_zoom_1 (Window *window)
-{
-	WindowPrivate *priv;
-	UIImage *ui;
-
-	g_return_if_fail (window != NULL);
-	g_return_if_fail (IS_WINDOW (window));
-
-	priv = window->priv;
-	g_return_if_fail (priv->mode == WINDOW_MODE_IMAGE);
-
-	g_assert (priv->content != NULL && IS_UI_IMAGE (priv->content));
-	ui = UI_IMAGE (priv->content);
-
-	ui_image_set_zoom (ui, 1.0);
-}
-
-/**
- * window_zoom_fit:
- * @window: An image window.
- * 
- * Zooms an image to fit the window size.
- **/
-void
-window_zoom_fit (Window *window)
-{
-	WindowPrivate *priv;
-	UIImage *ui;
-
-	g_return_if_fail (window != NULL);
-	g_return_if_fail (IS_WINDOW (window));
-
-	priv = window->priv;
-	g_return_if_fail (priv->mode == WINDOW_MODE_IMAGE);
-
-	g_assert (priv->content != NULL && IS_UI_IMAGE (priv->content));
-	ui = UI_IMAGE (priv->content);
-
-	ui_image_zoom_fit (ui);
+	return priv->ui;
 }
